import { Subscription } from "../Common/Subscription";
import type { Path, PathValue } from "./DotPath";
import { BaseStoreImpl, PatchOperationFunction, PatchOperationObject, StoreAction, StoreActionWithSubscriptions } from "./StoreTypes";
import type { BaseStoreClass, WatchFunction } from "./StoreTypes";
import { ClassStoreSymbol, DescriptorGroups } from "./StoreUtils";
export declare const InternalStoreKeys: string[];
export declare class BaseStore<TStore, TState> implements BaseStoreImpl<TStore, TState> {
    #private;
    private __originalState;
    private __state;
    private __stateWatcherFuncs;
    [ClassStoreSymbol]: any;
    __descriptors: DescriptorGroups;
    __handlers: Subscription;
    __actionHandlers: StoreActionWithSubscriptions<TStore, TState>[];
    __actionSubscriptions: {
        before: Subscription;
        after: Subscription;
        error: Subscription;
    };
    constructor();
    /**
     * Reset our state back to the original values defined in `get state()`
     */
    $reset(): void;
    /**
     * Access the watcher functions for each state key
     *
     * @template TState
     * @returns {{[K in keyof TState]: WatchFunction<TState[K]>}}
     */
    get $watch(): {
        [K in keyof TState]: WatchFunction<TState[K]>;
    };
    /**
     * Quickly access a state value via dot notation string
     *
     * @template TState
     * @param {P} path
     * @param defaultValue
     * @returns {PathValue<TState, P>}
     */
    $getState<P extends Path<TState>>(path: P, defaultValue?: any): PathValue<TState, P>;
    private __setState;
    /**
     * Apply a patch operation to the state
     *
     * @example
     * ```
     *  store.$patch({myStateValue: 'a new value', anotherStateValue: 'another new value'});
     *  store.$patch((state) => {
     *      state.myStateValue = 'a new value';
     *      state.anotherStateValue = 'another new value';
     *  });
     * ```
     *
     * @template TState
     * @param {PatchOperationFunction<TState>} patchOperation
     */
    $patch(patchOperation: PatchOperationFunction<TState>): void;
    $patch(patchOperation: PatchOperationObject<TState>): void;
    $onAction<K extends keyof TStore>(actionHandler?: (context: StoreAction<TStore, TState>) => void): () => void;
    addExtensions(extensions: {
        [key: string]: any;
    }[]): void;
    addExtension(extension: {
        [key: string]: any;
    }): void;
}
export declare function Store<TStore, TState>(): BaseStoreClass<TStore, TState>;
